// ~/jpHMM/src/alignment.h
/**************************************************************
 File name	: alignment.h
 Author		: Anne-Kathrin Schultz
 Version	: 2.0
 Date		: Oct 2010

Content:

 Classes: Alignment
          Seq
          Subtype

 Read in of information about 
  - subtypes
  - sequences 
  - reference genome sequence 
 in the alignment
***************************************************************/


#ifndef _ALIGNMENT_H
#define _ALIGNMENT_H
#include "std_header.h"

class Subtype;
class Seq;

class Alignment
{
   friend ostream& operator<<(ostream& s, const Alignment& al);
   public:
      Alignment(string infile, bool is_extended, int orig_al_length);
      ~Alignment();
      void check();
      Seq*             ref_seq()                      const;
      bool             has_ref_seq()                  const;
      int              ref_seq_pos_to_al_col(int pos) const;
      vector<Subtype*> subs()                         const {return m_subs;};
      bool             is_extended()                  const {return m_is_extended;};
      int              orig_len()                     const {return m_orig_len;};
      
  private:
      vector<Subtype*>  m_subs;
      vector<Seq*>      m_seqs;
      bool              m_has_ref_seq;
      Seq*              m_ref_seq;
      int               m_num_residues;
      int               m_num_considered_columns;
      bool              m_is_extended;  //! original alignment was extended (for circular genomes)
      int               m_orig_len;     //! = length of alignment for non-extended alignment (only extended for circular genomes)
};



class Seq
{
   friend ostream& operator<<(ostream& s, const Seq& seq);
   public:
      Seq(string name, Subtype* sub, string str);
      ~Seq(){};
      void check();

      Subtype* sub()      const {return m_sub;};
      string   name()           { check(); return m_name; };
      string   str()      const {return m_str;};
      char     str(int i) const {return m_str[i];};

   private:
      Subtype* m_sub;
      string   m_name;
      string   m_str;
};



class Subtype
{
   friend ostream& operator<<(ostream& s, const Subtype& st);
   public:
      Subtype(string cur_name);
      ~Subtype();
      void check();

      Seq*          seq(int i) const {return m_seqs[i];};
      vector<Seq*> &seqs()           {return m_seqs;};
      vector<Seq*>  seqs()     const {return m_seqs;};
      int           num_seqs() const {return m_seqs.size();};
      string        name()     const {return m_name;};
      int           len()      const {return m_len;};
      int          &len()            {return m_len;};
   private:
      string       m_name;
      vector<Seq*> m_seqs;
      int          m_len;
};

ostream& operator<<(ostream& s, const Alignment& al);
ostream& operator<<(ostream& s, const Subtype& st);
ostream& operator<<(ostream& s, const Seq& seq);

#endif  // _ALIGNMENT_H
