// ~/jpHMM/src/alphabet.h
/**************************************************************
 File name	: alphabet.h
 Author		: Anne-Kathrin Schultz
 Version	: 2.0
 Date		: Oct 2010

Content:

 Classes: Alphabet
          TransAlphabet

 Task: Determine nucleotide/amino acid and transition alphabet:
        = allowed nucleotides in alignment
	  and allowed transitions (e.g. M -> M)
***************************************************************/

#ifndef ALPHABET_H
#define ALPHABET_H

#include "std_header.h"

using namespace std;

class Alphabet
{
   friend ostream& operator<<(ostream& s, const Alphabet& al);
public:
  Alphabet(){};
  ~Alphabet(){};
  void        init(vector<char> characters, vector<char> all_characters);
  char        character(int i)                      const;
  string      characters(vector<int> v)             const;
  int         idx(char c)                           const;
  vector<int> indices(string s)                     const;
  bool        is_legal_indices(vector<int> v)       const;
  string      remove_illegal_characters(string str) const;
  bool        is_legal_character(char c)            const { return ((m_char_to_idx.find(c) != m_char_to_idx.end()) ? true : false); };
  bool        is_legal_idx(unsigned int i)          const { return (((i >= 0) && (i < m_characters.size())) ? true : false); };
  int         num_chars()                           const { return m_characters.size(); };
  void        check()                               const;

private:
  vector<char>   m_characters;
  map<char, int> m_char_to_idx;
  bool           m_is_init;

};

class TransAlphabet
{
   friend ostream& operator<<(ostream& s, const TransAlphabet& t_al);
public:
  TransAlphabet(){};
  ~TransAlphabet(){};
  void init(vector< pair<StateType, StateType> > transitions);
  pair<StateType, StateType> transition(int i) const;
  int  idx(StateType s, StateType t)           const;
  bool is_legal_pair(StateType s, StateType t) const;
  bool is_legal_indices(vector<int> v)         const;
  bool is_legal_idx(unsigned int i)            const {return (((i >= 0) && (i < m_transitions.size())) ? true : false); }
  int  num_trans()                             const {return m_transitions.size(); };
  void check()                                 const;

private:
  vector< pair<StateType, StateType> > m_transitions;
  map<pair<StateType, StateType>, int> m_pair_to_idx;
  bool                                 m_is_init;

};

ostream& operator<<(ostream& s, const Alphabet& al);
ostream& operator<<(ostream& s, const TransAlphabet& t_al);

#endif
