// ~/jpHMM/src/emissionPriors.cpp
/**************************************************************
 File name	: emissionPriors.cpp
 Author		: Anne-Kathrin Schultz
 Version	: 2.0
 Date		: Oct 2010

Content:

 Read in of priors (pseudo counts) for emission probabilities
***************************************************************/

#include "std_source.h"
#include "error.h"
#include "const_def.h"
#include "emissionPriors.h"


using namespace std;


/*******************************************************\

 *                 class EmissionPriors                *

\*******************************************************/

//-------------------------------------------------------
// EmissionPriors():
//-------------------------------------------------------
EmissionPriors::EmissionPriors(string infile) {

  m_match_priors.clear();
  m_insert_priors.clear();

  bool has_match_priors  = false;
  bool has_insert_priors = false;
  string str = "";

  ifstream ifstrm(infile.c_str());

  if (!ifstrm) {
    stringstream ss;
    ss << "Failure of the emission priors input file: ";
    ss <<  infile;
    throw Error(ss.str());
  }
   // throw Error("Failure of the emission priors input file.");

  while (!ifstrm.eof()) {
    getline(ifstrm, str);
    if (str.substr(0, 1) == "#") {
      if (str.find("match") != string::npos) {
        has_match_priors  = true;
        has_insert_priors = false;
      }
      else if (str.find("insert") != string::npos) {
        has_match_priors  = false;
        has_insert_priors = true;
      }
      else {
        has_match_priors  = false;
        has_insert_priors = false;
      }
    }
    else if (!str.empty()) { // ws
      if (has_match_priors || has_insert_priors) {
        size_t found = str.find_first_of("0123456789");
        if (found != string::npos) {
          stringstream ss(str.substr(found));

          mytype cur_prior;
          ss >> cur_prior;

          if (has_match_priors)
            m_match_priors += cur_prior;
          else if (has_insert_priors)
            m_insert_priors += cur_prior;
        }
      }
    }
  }

  //! delete last entry (= sum of priors):
  m_match_priors.pop_back();
  m_insert_priors.pop_back();

  check();
}

//------------------------------------------------------------------------------
// void check():
//
//------------------------------------------------------------------------------
void EmissionPriors::check(){

  if (m_match_priors.size() != NUM_NUCL || m_insert_priors.size() != NUM_NUCL)
    throw Error("some of the match and insert priors for dna sequences are missing.");
}

//------------------------------------------------------------------------------
// output:
//------------------------------------------------------------------------------
ostream& operator<<(ostream& s, const EmissionPriors& e) {
  s << "match: " << e.m_match_priors << endl;
  s << "insert: " << e.m_insert_priors << endl;
  return s;
}
