// ~/jpHMM/src/rewrite_jpHMM/extEmTrProbs.h
/**************************************************************
 Dateiname	: extEmTrProbs.cpp
 Autor		: Ingo Bulla
 Version	: 2.0
 Datum		: May 2010

 Inhalt:
 Klassen: ExtEmTrProbs
 ***************************************************************/

#include "std_source.h"
#include "extEmTrProbs.h"
#include "error.h"
#include "const_def.h"

using namespace std;


//--------------------------------------------------------------------------
// constructor
//--------------------------------------------------------------------------
ExtEmTrProbs::ExtEmTrProbs(string em_probs_filename, string tr_probs_filename, string cons_file_name)
{
	read_probs(&m_em_probs, em_probs_filename, NUM_NUCL);
	read_probs(&m_tr_probs, tr_probs_filename, NUM_TRANS);
	fstream probs_strm(cons_file_name.c_str());
	if (!probs_strm)
		throw Error("could not open consensus columns file");
	while (!probs_strm.eof()) {
		unsigned x;
		probs_strm >> x;
		m_cons_cols += x;
	}
}


//--------------------------------------------------------------------------
// read_probs(map< unsigned, map<unsigned, vector<double> > >* probs, string probs_filename);
// reads probabilities from file
//--------------------------------------------------------------------------
void ExtEmTrProbs::read_probs(map< string, map<unsigned, vector<double> > >* probs, string probs_filename, unsigned num_probs)
{
	string str;
	fstream probs_strm(probs_filename.c_str());
	if (!probs_strm) {
		stringstream ss;
		ss << "could not open " << probs_filename;
		throw Error(ss.str());
	}
	while (getline(probs_strm, str)) {
		char_separator<char> sep(" ");
		tokenizer< char_separator<char> > toknzr(str, sep);
		vector<string> tokens;
		for(tokenizer< char_separator<char> >::iterator it = toknzr.begin(); it != toknzr.end(); ++it)
			tokens += *it;
		if (tokens.size() != num_probs + 2) {
			stringstream ss;
			ss << "Wrong number of tokens in external probabilities (actual: " << tokens.size() << ", expected: " << num_probs + 2 << " )" << endl
			   << "Filename: " << probs_filename << endl
			   << "Line: " << str << endl;
			throw Error(ss.str());
		}
		
		unsigned pos;
		string sub;
		vector<double> curr_probs;
		try {
			pos = lexical_cast<unsigned>(tokens[0]);
			sub = tokens[1];
			for (unsigned i = 2; i < num_probs + 2; ++i)
				curr_probs += lexical_cast<double>(tokens[i]);
		}
		catch (bad_lexical_cast& e) {
			throw Error("Type error in external probabilities");
		}

		if (probs->find(sub) == probs->end()) {
			(*probs)[sub] = map<unsigned, vector<double> >();
		}
		(*probs)[sub][pos] = curr_probs;
	}
}


//--------------------------------------------------------------------------
// output operator
//--------------------------------------------------------------------------
ostream& operator<<(ostream& s, const ExtEmTrProbs& eetp)
{
	s << "em_probs:" << endl;
	FOREACH(pair< string COMMA map<unsigned COMMA vector<double> > > p_sub, eetp.m_em_probs) {
		FOREACH(pair< unsigned COMMA vector<double> > p_pos, p_sub.second) {
			s << p_sub.first << " " << p_pos.first << " ";
			FOREACH (double prob, p_pos.second)
				s << prob << " ";
			s << endl;
		}
	}
	
	return s;
}


