#include <map>
#include <string>
#include <sstream>
#include <stdexcept>
#include <vector>
#include <boost/foreach.hpp>
#include "html_cell.h"
#include "html_tab.h"

using namespace std;
using namespace boost;

#define FOREACH BOOST_FOREACH
#define COMMA ,


Html_Cell::Html_Cell(Cell_Type type, unsigned row, unsigned col)
: m_row(row), m_col(col), m_type(type), m_use_width(false), m_bg_color(""),  m_font_color("black"),  m_font_weight(FW_NORMAL), m_font_size(FS_NORMAL),
  m_align(CENTER), m_col_span(1)
{ }

/**
 * generate HTML output of the cell
 * @param vert_line a vertical line to be attached to the right of the cell
 * @return the output
 */
string Html_Cell::output(const Html_Tab& tab)
{
	stringstream ret;
	string main_tag = (m_type == HEAD) ? "th" : "td"; 
	ret << "<" + main_tag;

	unsigned style_counter = 0;
	const unsigned MAX_NUM_STYLES = 10;
	stringstream styles[MAX_NUM_STYLES];
	// width
	if (m_use_width)
		styles[style_counter] << "width:" << m_width << "px";
	// font color
	if (m_font_color != "black")
		styles[++style_counter] << "color:" << m_font_color;
	// background color
	if (!m_bg_color.empty())
		styles[++style_counter] << "background-color:" << m_bg_color;
	else
		if ((m_type == NORM) && tab.has_row_color(m_row))
			styles[++style_counter] << "background-color:" << tab.row_color(m_row);
	// font weight
	switch (m_font_weight) {
	case FW_NORMAL:
		break;
	case FW_BOLD:
		styles[++style_counter] << "font-weight:bold";
		break;
	}
	// font size
	switch (m_font_size) {
	case FS_NORMAL:
		break;
	case FS_LARGE:
		styles[++style_counter] << "font-size:large";
		break;
	}
	// border
	if (tab.has_vert_line(m_col))
		styles[++style_counter] << "border-right:1px solid #000";
	ret << " style=\"";
	for (unsigned i = 0; i < MAX_NUM_STYLES; ++i) {
		if (!styles[i].str().empty()) {
			if (i)
				ret << "; ";
			ret << styles[i].str();
		}
	}
	ret << "\"";

	// align
	ret << " align=\"";
	switch (m_align) {
	case CENTER:
		ret << "center";
		break;
	case RIGHT:
		ret << "right";
		break;
	}
	ret << "\"";
	// colspan
	if (m_col_span > 1)
		ret << " colspan=\"" << m_col_span << "\"";

	ret << "> ";
	ret << ((m_str == "") ? "&nbsp" : m_str) << " </" << main_tag << ">";
	return ret.str();
}



void Html_Cell::set_col_span(unsigned col_span)
{
	if (!col_span)
		throw runtime_error("trying to set col_span to zero");
	m_col_span = col_span;
}


