/**
 * \file html_cell.h
 */

#ifndef _HTMLCELL_H
#define _HTMLCELL_H

#include <string>
#include <boost/shared_ptr.hpp>

enum Cell_Type { HEAD, NORM };
enum Align_Type { CENTER, RIGHT };
enum Font_Weight_Type { FW_NORMAL, FW_BOLD };
enum Font_Size_Type { FS_NORMAL, FS_LARGE };

struct Html_Tab;



/**
 * a cell of Html_Tab
 */
class Html_Cell 
{
public:
	Html_Cell(Cell_Type type, unsigned row, unsigned col);
	std::string output(const Html_Tab& tab);
	
	unsigned row() const { return m_row; }
	unsigned col() const { return m_col; }
	std::string& str() { return m_str; }
	std::string str() const { return m_str; }
	Cell_Type type() const { return m_type; }
	void set_width(unsigned width) { m_use_width = true; m_width = width; }
	bool& use_width() { return m_use_width; }
	bool use_width() const { return m_use_width; }
	std::string& bg_color() { return m_bg_color; }
	std::string bg_color() const { return m_bg_color; }
	std::string& font_color() { return m_font_color; }
	std::string font_color() const { return m_font_color; }
	void set_bold() { m_font_weight = FW_BOLD; }
	void set_large() { m_font_size = FS_LARGE; }
	void set_align_center() { m_align = CENTER; }
	void set_align_right() { m_align = RIGHT; }
	void set_col_span(unsigned col_span);
	unsigned col_span() const { return m_col_span; }
private:
	unsigned m_row;
	unsigned m_col;
	std::string m_str;
	Cell_Type m_type;
	unsigned m_width;
	bool m_use_width;
	std::string m_bg_color;
	std::string m_font_color;
	Font_Weight_Type m_font_weight;
	Font_Size_Type m_font_size;
	Align_Type m_align;
	unsigned m_col_span;
};

#endif  // HTMLCELL_H
