// ~/jpHMM/src/query.cpp
/**************************************************************
 File name	: query.cpp
 Author		: Anne-Kathrin Schultz
 Version	: 2.0
 Date		: Oct 2010

Content:

 For each query sequence
  - string of nucleotides/amino acids
  - information about name, number in input file, full-length or fragmental sequence
  - for extended circular sequences, original start and end position in extended sequence
  - if information is given: start and end position of the sequence wrt the given alignment
***************************************************************/

#include "std_source.h"
#include "error.h"
#include "alphabet.h"
#include "const_def.h"
#include "query.h"

using namespace std;

/*******************************************************\

 *                 class Query                         *

\*******************************************************/

/***********************************************************************************************************************//**
  Query.
  
  Create query object for current query sequence.

   @param[in] name Name of query sequence.
   @param[in] no Number/position of current query sequence in list of all query sequences.
   @param[in] str Query sequence (string of nucleotides).
   @param[in] start_in_al Start position of query sequence with respect to given alignment.
   @param[in] end_in_al End position of query sequence with respect to given alignment.
   @param[in] is_full_length_seq Query sequence is full-length sequence (else: fragment).
   @param[in] ext Number of nucleotides the query sequence has been extended.
   @param[in] start_in_ext_q_str Start position of original query sequence in the extended query sequence.
   @param[in] end_in_ext_q_str End position of original query sequence in the extended query sequence.
*****************************************************************************************************************************/
Query::Query(string name, int no, string str, int start_in_al, int end_in_al, bool is_full_length_seq, int ext, int start_in_ext_q_str, int end_in_ext_q_str) {
  
  m_name          = name;
  m_no            = no;
  m_str_incl_gaps = str;
  m_str           = "";
  m_is_full_length_seq = is_full_length_seq;  //! necessary for circular genomes: is full-length sequence (or fragment)?
  m_start_in_al   = start_in_al;
  m_end_in_al     = end_in_al;
  m_ext           = ext;
  m_start_in_ext_q_str = start_in_ext_q_str;
  m_end_in_ext_q_str   = end_in_ext_q_str;
  m_prog_version  = Const_Def::prog_version();
  
  int num_gaps = 0;
  for (unsigned int i=0; i<m_str_incl_gaps.length(); ++i){
    if (m_str_incl_gaps[i] != '-' && m_str_incl_gaps[i] != '*' && m_str_incl_gaps[i] != '.') {
      m_str += m_str_incl_gaps[i];
      m_list_of_gaps += num_gaps;
    }
    else {
      num_gaps++;
    }
  }

  //! for last query position replace last entry in list_of_gaps by no. of all gaps in query
  //! -> for last query position count also gaps at the end of query (important if query ends with a gap):
  m_list_of_gaps.back() = num_gaps;
  m_num_gaps            = num_gaps;
}

/***************************//**
  Check query.
*******************************/
void Query::check(){

}

/***************************************//**
  Output of query sequence information.
*******************************************/
ostream& operator<<(ostream& s, const Query& q) {
  s << "name: " << q.m_name << endl;	
  s << "str: " << q.m_str << endl;	
  s << "start in ref seq: " << q.m_start_in_al << endl;	
  s << "end in ref seq: " << q.m_end_in_al << endl;	
  return s;
}

