// ~/jpHMM/src/query.h
/**************************************************************
 File name	: query.h
 Author		: Anne-Kathrin Schultz
 Version	: 2.0
 Date		: Oct 2010

Content:

 Classes: Query

 For each query sequence
  - string of nucleotides/amino acids
  - information about name, number in input file, full-length or fragmental sequence
  - for extended circular sequences, original start and end position in extended sequence
  - if information is given: start and end position of the sequence wrt the given alignment
***************************************************************/

#ifndef _QUERY_H
#define _QUERY_H
#include "std_header.h"

using namespace std;

class Query
{
   friend ostream& operator<<(ostream& s, const Query& q);
   public:
   Query(string name, int no, string str, int start, int end, bool is_full_length_seq, int ext, int start_in_ext_q_str, int end_in_ext_q_str);
     ~Query(){};
     void check();
     string name()                         { check(); return m_name; };
     int    no()                     const { return m_no; };
     string str()                    const { return m_str; };
     char   str(int i)               const { return m_str[i]; };
     string str_incl_gaps()          const { return m_str_incl_gaps; };
     char   str_incl_gaps(int i)     const { return m_str_incl_gaps[i]; };
     bool   is_full_length_seq()     const { return m_is_full_length_seq; };
     int    start_in_al()            const { return m_start_in_al; };
     int   &start_in_al()                  { return m_start_in_al; };
     int    end_in_al()              const { return m_end_in_al; };
     int   &end_in_al()                    { return m_end_in_al; };
     int    ext()                    const { return m_ext; };
     int    start_in_ext_q_str()     const { return m_start_in_ext_q_str; }; //! position of original first (start) position in extended query string (only for circular genomes)
     int    end_in_ext_q_str()       const { return m_end_in_ext_q_str; };
     Program_Version prog_version()  const { return m_prog_version; }; 
     Program_Version &prog_version()       { return m_prog_version; };
     mytype beam_width()             const { return m_beam_width; };
     mytype &beam_width()                  { return m_beam_width; };
     vector<int> list_of_gaps()      const { return m_list_of_gaps; };
     int    num_gaps(int i)          const { return m_list_of_gaps[i]; };

     
   private:
     string m_name;
     int    m_no;                    //! number in list of all queries
     string m_str_incl_gaps;
     string m_str;
     int    m_start_in_al;           //! most left column in alignment that query can be aligned to
     int    m_end_in_al;             //! most right column in alignment that query can be aligned to
     bool   m_is_full_length_seq;    //! for circular: sequence is a full_length sequence (or fragment)?
     int    m_ext;                   //! extension of query, if circular and full_length genome
     int    m_start_in_ext_q_str;    //! for circular, full_length genomes: start of original sequence in extended sequence string
     int    m_end_in_ext_q_str;      //! for circular, full_length genomes: end of original sequence in extended sequence string
     Program_Version m_prog_version; 
     mytype m_beam_width;
     vector<int>  m_list_of_gaps;    //! number of preceding gaps (=num_gaps) for each qp
     int    m_num_gaps;
};

ostream& operator<<(ostream& s, const Query& q);

#endif  // _QUERY_H
