// ~/jpHMM/src/transitionPriors.cpp
/**************************************************************
 File name	: transitionPriors.cpp
 Author		: Anne-Kathrin Schultz
 Version	: 2.0
 Date		: Oct 2010

Content:

 Read in of priors (pseudo counts) for transition probabilities
***************************************************************/

#include "std_source.h"
#include "error.h"
#include "const_def.h"
#include "transitionPriors.h"


using namespace std;


/*******************************************************\

 *                 class TransitionPriors              *

\*******************************************************/

//------------------------------------------------------------------------------
// TransitionPriors():
//------------------------------------------------------------------------------
TransitionPriors::TransitionPriors(string infile) {
  bool   has_trans_priors = false;
  string str              = "";

  ifstream ifstrm(infile.c_str(), ios::in);

  if (!ifstrm) {
    cout << infile << endl;
    throw Error("Failure of the transition priors input file.");
  }

  while (!ifstrm.eof()) {
    getline(ifstrm, str);
    if (str.substr(0, 1) == "#") {
      if (str.find("transition") != string::npos)
        has_trans_priors = true;
      else
        has_trans_priors = false;
    }
    else if (!str.empty()) { // ws
      if (has_trans_priors) {
        size_t found = str.find_first_of("0123456789");
        if (found != string::npos) {
          stringstream ss(str.substr(found)); 
          mytype cur_prior;

          ss >> cur_prior;
          m_trans_priors += cur_prior;
        }
      }
    }
  }
  check();
}

//------------------------------------------------------------------------------
// void check():
//
//------------------------------------------------------------------------------
void TransitionPriors::check(){
   if (m_trans_priors.size() != NUM_TRANS) 
     throw Error("The given number of state transitions within a profile HMM for one column is not correct.");
}

//------------------------------------------------------------------------------
// output:
//------------------------------------------------------------------------------
ostream& operator<<(ostream& s, const TransitionPriors& t) {
  s << t.m_trans_priors << endl;
  return s;
}
